# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2010 Nexedi SA and Contributors. All Rights Reserved.
# Copyright (c) 2006-2008 Zope Corporation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

import os
import logging

#import zc.buildout
import zc.buildout.easy_install
import zc.buildout.download
import zc.recipe.egg
import subprocess

class Recipe(object):
    
  def __init__(self, buildout, name, options):
    self.buildout, self.options, self.name = buildout, options, name
    self.logger=logging.getLogger(self.name)
    self.egg = zc.recipe.egg.Egg(buildout, options['recipe'], options)


    options.setdefault('location',os.path.join(
        buildout['buildout']['parts-directory'],
        self.name,
        ))

    options.setdefault('kvm_bin_directory',
                       buildout['buildout']['bin-directory'])

    options.setdefault('kvm_run_directory',
                       buildout['buildout']['var-directory'])

    # File is preferred, if defined image will be ignored.
    options.setdefault("image", '')
    options.setdefault("file", '')

    options.setdefault("image_type", "drive")

    options.setdefault("md5sum", '')
    options.setdefault("gzip", 'true')
    options.setdefault("gzip_md5", '')

    options.setdefault("kvm_vnc", "")
    options.setdefault("kvm_graphic", "")
    options.setdefault("kvm_redir", "")
    options.setdefault("kvm_net", "")

  def _writeSSHKey(self, key, key_path):
    """ Write the SSK key into the file
    """
    if not os.path.exists(key_path):
      f = file(key_path, 'w')
      f.write(key)
      f.close()
      os.chmod(key_path, 0700)
    
  def install(self):
    options = self.options
    location = options['location']
    if not os.path.exists(location):
      os.mkdir(location)

    if options.get('file') in ["", None ]:
      download_file, is_temp = self.download()
      image = self.copy(download_file)
    else:
      image = options.get('file')

    image_type = options.get("image_type")
    if image_type == "drive":
      kvm_drive = "-drive file=" + ','.join([image, "if=virtio", "cache=none", "boot=on"])
    else:
      kvm_drive = "-%s %s" % (image_type, image)

    kvm_redir = options.get('kvm_redir').split()
    requirements, ws = self.egg.working_set(['erp5.recipe.kvm'])
    if options.get("ssh_port", "") not in ["", None ]:
      ssh_port = options.get("ssh_port")
      ssh_hostname = options.get("ssh_hostname", "localhost")
      ssh_key_path = options.get("ssh_key_path")
      ssh_key = options.get("ssh_key", "")
      if ssh_key not in [ None, "" ]:
        self._writeSSHKey(ssh_key, ssh_key_path)
      ssh_user = options.get("ssh_user", "root")

      kvm_redir.append(" tcp:%s::22 " % ssh_port)

      script_name = "%s-sendfile" % self.name
      scripts = zc.buildout.easy_install.scripts(
          [(script_name,'erp5.recipe.kvm.ssh', 'put')],
          ws, options['executable'], options.get("kvm_bin_directory"),
          arguments = ("\n        '%s', '%s',"
                       "\n        '%s', '%s', "
                       "\n        sys.argv[1], sys.argv[2]" % (ssh_user, ssh_hostname,
                                                               ssh_port, ssh_key_path)))
      
      script_name = "%s-getfile" % self.name
      scripts = zc.buildout.easy_install.scripts(
          [(script_name,'erp5.recipe.kvm.ssh', 'get')],
          ws, options['executable'], options.get("kvm_bin_directory"),
          arguments = ("\n        '%s', '%s',"
                       "\n        '%s', '%s', "
                       "\n        sys.argv[1], sys.argv[2]" % (ssh_user, ssh_hostname,
                                                               ssh_port, ssh_key_path)))
  
      script_name = "%s-runscript" % self.name
      scripts = zc.buildout.easy_install.scripts(
          [(script_name,'erp5.recipe.kvm.ssh', 'run')],
          ws, options['executable'], options.get("kvm_bin_directory"),
          arguments = ("\n        '%s', '%s',"
                       "\n        '%s', '%s', "
                       "\n        sys.argv[1]" % (ssh_user, ssh_hostname,
                                                  ssh_port, ssh_key_path)))
    script_name = "%s-ctl" % self.name
    # XXX The followed code requires some improvements
    snapshot = "False"
    kvm_snapshot = options.get("kvm_snapshot", "true").lower()
    if kvm_snapshot == 'true': 
      snapshot = "True"

    kvm_pid_file = "%s/%s.pid" % (options.get('kvm_run_directory'), self.name)
    scripts = zc.buildout.easy_install.scripts(
        [(script_name,'erp5.recipe.kvm.kvm', 'ctl')],
        ws, options['executable'], options.get("kvm_bin_directory"),
        arguments = ("\n        '%s', "
                     "\n        '%s', "
                     "\n        %s, "
                     "\n        %s, "
                     "\n         %s , "
                     "\n        '%s', "
                     "\n        '%s', "
                     "\n        sys.argv[1]" % (
                                   kvm_drive, 
                                   kvm_pid_file,
                                   options.get('kvm_net').split(),
                                   kvm_redir,
                                   snapshot, 
                                   options.get("kvm_vnc"), 
                                   options.get("kvm_graphic"))))
    # XXX Improvements until here
    return [] # instance related recipe, it can be disaster to allow buildout
              # to remove instances

  def download(self):
    """Download image file
    """
    url = self.options['image']
    namespace = self.options['recipe']
    download = zc.buildout.download.Download(self.buildout['buildout'],
                                             namespace=namespace,
                                             logger=self.logger)
    return download(url, self.options.get("md5_download", None))

  def copy(self, download_file):
    """ Copy downloaded file to a appropriate place
    """
    image = os.path.join(self.options['location'], self.name)

    if os.path.exists(image):
      self.logger.info("Image (%s) already exists... "
                       "skipping uncompress." % image)
      return image

    if self.options["gzip"].lower() == "true":
      self._gunzip(download_file, image)
      return self._checkMD5(image)

    os.copy(download_file, image)
    return self._checkMD5(image)

  def _checkMD5(self, image):
    """ Verify MD5.
    """
    options = self.options
    md5_image = options.get("md5_image", None)
    assert zc.buildout.download.check_md5sum(image, md5_image) == True
    return image
    
  def _gunzip(self, download_file, image):
    """ Use gunzip command to uncompress image.
    """
    command = 'gunzip -cd %s > %s' % (download_file, image)
    code = subprocess.call(command, shell=True)
    assert code == 0
    
  update = install
