import subprocess
import os
import signal

BASE_COMMAND = "qemu-kvm %s %s -k fr -m 1024 %s -daemonize -pidfile %s %s "

class Kvm:
  def __init__(self, drive, pid_file, net, redir, snapshot=True, vnc=None, graphic=None):
    """ Handle the KVM command for a certain image, and 
        start ssh in a certain port.

	image: Image Path
    """
    # XXX Make drive configuration configurable
    self.drive = drive
    self.net = ''.join([" -net %s " % n for n in net ])
    self.redir = ''.join([" -redir %s " % r for r in redir])

    self.kvm_pid_file = pid_file
    self.kvm_args = ""
    if vnc not in [None, ""]:
      self.kvm_args += " -vnc %s " % vnc

    if graphic not in [None, ""]:
      self.kvm_args += " -vga %s " % (graphic)
    else:
      self.kvm_args += " -nographic "

    if snapshot:
      self.kvm_args += " -snapshot "


  def start(self):
    """ Start Kvm in background."""
    command = BASE_COMMAND % ( self.net,
                               self.redir,
                               self.drive,
                               self.kvm_pid_file, 
                               self.kvm_args)
    print command
    return_code = subprocess.call(command, shell=True)
    assert return_code == 0

  def stop(self):
    """ Stop Kvm in the server """
    try:
      pid = int(open(self.kvm_pid_file, 'r').read().strip())
    except:
      pid = None
      print "Can't read child pidfile %s!" % self.kvm_pid_file
      return
    # XXX SIGKILL is too brutal?
    os.kill(pid, signal.SIGKILL)

  def status(self):
    """ Check if Kvm process is running.
    """
    try:
      pid = int(open(self.kvm_pid_file, 'r').read().strip())
    except:
      pid = None
      print "Can't read child pidfile %s!" % self.kvm_pid_file
      return False
    try:
      os.kill(pid , 0)
    except OSError, err:
      return False
    return True

  def restart(self):
    """ Stop and starts the KVM 
    """
    self.stop()
    self.start()

def ctl(drive, pid_file, net, redir, 
        snapshot, vnc, graphic, command):
  """ Control KVM Command
  """
  handler = Kvm(drive, pid_file, net, redir, snapshot, vnc, graphic)
  command = getattr(handler, command, None)
  if command is not None:
    print command()
    return

  print "ERROR!"
  return "ERROR!"
