
import paramiko
from paramiko.ssh_exception import SSHException
import os

class connection(object):
  """Holds an SSH connection to a remote SSH server.
  """
  def __init__(self, username, host, port, key_path):
    self.username = username
    self.host = host
    self.port = port
    self.password = ''
    self.transport = None
    self.sftp = None
    if os.path.exists(key_path):
      self.key_path = key_path
    else:
      raise ValueError, 'key_path does not exist: %s' % key_path

  def connect(self):
    """ Connect and instantiate transport and sftp.

        XXX It could be supported Password if key is not
	    defined.
    """
    self.transport = paramiko.SSHClient()
    self.transport.load_system_host_keys()
    self.transport.set_missing_host_key_policy(
		    paramiko.AutoAddPolicy())
    try:
      key = paramiko.RSAKey.from_private_key_file(self.key_path)
    except SSHException, e:
      key = paramiko.DSSKey.from_private_key_file(self.key_path)
    
    try:
      self.transport.connect(hostname=str(self.host), 
		             port=int(self.port), 
		             username=self.username, 
			     timeout=180,
                             pkey=key, look_for_keys=False)
    except SSHException, e:
      self.transport.close()
      raise SSHException(str(e))

  def close(self):
    """Close the sftp and transport connection.
    """
    self.transport.close()

  def test(self):
    """ Test connect to a remote server.
    """
    try:
      try:
        self.connect()
        return True
      except SSHException, e:
        return False
    finally:
      self.close()

  def get(self, remote_path, local_path):
    """ Get a remote file and transfer for local file 
    """
    try:
      self.connect()
      self.transport.open_sftp().get(remote_path, local_path)
    finally:
      self.close()
      
  def put(self, remote_path, local_path):
    """ Send a local file to a remote path
    """
    try:
      self.connect()
      self.transport.open_sftp().put(local_path, remote_path)
    finally:
      self.close()

  def run(self, command):
    """ Run a command in remote server
    """
    response = ""
    try:
      self.connect()
      stdin, stdout, stderr = self.transport.exec_command(command + " 2>&1 ")
      out = stdout.read()
      err = stderr.read()
    finally:
      self.close()
    return out, err

def put(username, host, port, key_path, remote_path, local_path):
  """ Send file using paramiko
  """
  handler = connection(username, host, port, key_path)
  handler.put(remote_path, local_path)
  return True

def run(username, host, port, key_path, command, use_stdout=1):
  """ Connect into a remote instance and run a script
  """
  handler = connection(username, host, port, key_path)
  output , error = handler.run(command)
  if use_stdout:
    print output
    if len(error) > 0:
      print "="*79
      print error
      print "="*79
  return output, error
  
def get(username, host, port, key_path, remote_path, local_path):
  """ Get file by ssh 
  """
  handler = connection(username, host, port, key_path)
  handler.get(remote_path, local_path)
  return True
